#include <FL/Fl_Dial.H>
#include <FL/Fl_Gl_Window_Group.H>
#include <FL/Fl_Input_Choice.H>
#include <FL/Fl_Multiline_Input.H>
#include <FL/Fl_Round_Clock.H>
#include <FL/Fl.H>
#include <FL/forms.H>	// for fl_gettime()
#include <FL/gl.h>

class My_Gl_Window_Group : public Fl_Gl_Window_Group {
public:
  My_Gl_Window_Group(int x, int y, int w, int h, const char *l = 0)
  : Fl_Gl_Window_Group(x, y, w, h, l) {}

protected:
  void draw_gl(void) {
    long sec = 0, usec = 0;
    fl_gettime(&sec, &usec);
    int k = 100 * (sec % 10) + usec / 10000;
    int wi = k * w() / 1000;
    int hi = k * h() / 1000;

    glClearColor(0.0, 0.0, 0.0, 1.0);
    glClear(GL_COLOR_BUFFER_BIT);

    ortho();
    glColor3f(1.0, 0.0, 0.0);
    glBegin(GL_QUADS);
    glVertex2d(wi, 0);
    glVertex2d(w(), hi);
    glVertex2d(w() - wi, h());
    glVertex2d(0, h() - hi);
    glEnd();
  }
};

int main(int argc, char **argv) {
  Fl_Window *win = new Fl_Window(640, 480, "gl_group");
  Fl_Gl_Window_Group *group = new My_Gl_Window_Group(20, 20, 600, 440);

  Fl_Round_Clock *clock = new Fl_Round_Clock(20, 20, 200, 200);

  Fl_Input_Choice *inchoice = new Fl_Input_Choice(240, 20, 340, 24);
  inchoice->add("One, two, buckle my shoe");
  inchoice->add("Three, four, shut the door");
  inchoice->add("Five, six, pick up sticks");
  inchoice->add("Seven, eight, lay them straight");

  Fl_Multiline_Input *mlinput = new Fl_Multiline_Input(240, 64, 340, 156);
  mlinput->value(
    "The red quadrilateral in the background is\n"
    "drawn using OpenGL. It is redrawn whenever a\n"
    "child widget in the Fl_Gl_Window_Group is\n"
    "updated, which is why it moves intermittently.\n"
    "\n"
    "Note that non-rectangular widgets, like the\n"
    "clock, are currently not drawn properly by\n"
    "Fl_Gl_Window_Group."
  );

  Fl_Dial *dial = new Fl_Dial(20, 240, 100, 100);

  group->end();
  win->end();
  win->show();

  return Fl::run();
}
